<?php
/**
 * Class that manages all the features of Post Star Rating Wordpress plugin
 *
 */
class PSR {
	var $_points = 0;
	var $_user;
	
	/**
	 * Create the database tables to support plugin behaviour.
	 *
	 * @param boolean $echo If true echoes messages to user
	 */
	function install($echo = false) {
		global $table_prefix, $wpdb;

		$table_name = $table_prefix . "psr_post";
		if($wpdb->get_var("SHOW TABLES LIKE '{$table_name}'") !== $table_name) {
			$sql = "CREATE TABLE {$table_name} (
			  ID bigint(20) unsigned NOT NULL default '0',
			  votes int(10) unsigned NOT NULL default '0',
			  points int(10) unsigned NOT NULL default '0',
			  PRIMARY KEY (ID)
			);";

			require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
			dbDelta($sql);
			if ($echo) _e("Se ha creado la tabla de valoracin\n");
		} else {
			if ($echo) _e("La tabla de valoracin ya estaba creada\n");
		}

		$table_name = $table_prefix . "psr_user";
		if($wpdb->get_var("SHOW TABLES LIKE '{$table_name}'") !== $table_name) {
			$sql = "CREATE TABLE {$table_name} (
			  user varchar(32) NOT NULL default '',
			  post bigint(20) unsigned NOT NULL default '0',
			  points int(10) unsigned NOT NULL default '0',
			  PRIMARY KEY (user,post)
  		);";
			require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
			dbDelta($sql);
			if ($echo) _e("Se ha creado la tabla de puntuaciones\n");
		} else {
			if ($echo) _e("La tabla de puntuaciones ya estaba creada\n");
		}
	}
	
	/**
	 * Get the html that shows the stars for voting
	 * If the user has already vote then it shows stars with puntuation. No voting is allowed
	 *
	 * @return string
	 */
	function getVotingStars() {
		global $id, $wpdb, $table_prefix;
		$rated = false;
		if (isset($this->_user)) {
			$user = $wpdb->escape($this->_user);
			$table_name = $table_prefix . "psr_user";
			$rated = (bool) $wpdb->get_var("SELECT COUNT(*) FROM {$table_name} WHERE user='{$user}' AND post={$id}");
		}
		if (($this->_points > 0) && !$rated) {
			$user = $wpdb->escape($this->_user);
			$table_name = $table_prefix . "psr_user";
			$wpdb->query("INSERT INTO {$table_name} (user, post, points) VALUES ('{$user}', {$id}, {$this->_points})");
			$table_name = $table_prefix . "psr_post";
			if ($wpdb->get_var("SELECT COUNT(*) FROM {$table_name} WHERE ID={$id}")) {
				$wpdb->query("UPDATE {$table_name} SET votes=votes+1, points=points+{$this->_points} WHERE ID={$id};");
			} else {
				$wpdb->query("INSERT INTO {$table_name} (ID, votes, points) VALUES ({$id}, 1, {$this->_points});");
			}
			$rated = true;
		}
		if ($rated) {
			$html = $this->getStars();
		} else {
			$url = get_settings('home') . "/?p={$id}";
			$html = '<form action="' . $_SERVER['PHP_SELF'] . '" method="post" class="PSR_stars" onmouseout="PSR_star_out(this)"> ';
			for ($i = 1; $i <= 5; ++$i) {
				$html .= sprintf('<input type="radio" id="psr_star_%1$d" class="star" name="psr_stars" value="%1$d" onclick="this.form.submit();return true;" /><label class="PSR_no_star" for="psr_star_%1$d" onmouseover="PSR_star_over(this, ' . $i . ')">%1$d</label> ', $i);
			}
			$html .= '<input type="hidden" name="p" value="' . $id . '" />';
			$html .= '<input type="submit" name="vote" value="' . __('Votar') . '" />';
			$html .= '</form>';
		}
		return $html;
	}
	
	/**
	 * Get the html that shows the stars with puntuation.
	 *
	 * @return string
	 */
	function getStars() {
		global $id, $wpdb, $table_prefix;
		$table_name = $table_prefix . "psr_post";
		$data = $wpdb->get_row("SELECT votes, points FROM {$table_name} WHERE ID={$id}");
		if ($data->votes > 0) {
			$rate = $data->points / $data->votes;
		} else {
			$rate = 0;
		}
		$html = '<div class="PSR_stars"> ';
		for ($i = 1; $i <= 5; ++$i) {
			if ($i <= $rate) {
				$class = 'PSR_full_star';
				$char = '*';
			} elseif ($i <= ($rate + .5)) {
				$class = 'PSR_half_star';
				$char = '&frac12;';
			} else {
				$class = 'PSR_no_star';
				$char = '&nbsp;';
			}
			$html .= '<span class="' . $class . '">' . $char . '</span> ';
		}
		$html .= '<span class="PSR_votes">' . (int) $data->votes . '</span> <span class="PSR_tvotes">' . __('votos') . '</span>';
		$html .= '</div>';
		return $html;
	}
		
	/**
	 * Initialize the values. 
	 * Get the puntuation from url and the user from the cookies.
	 * If no cookie exists generate a new user.
	 * Refresh the cookie to hold the value of user for 1 year
	 *
	 */
	function init() {
		if (isset($_COOKIE['wp_psr'])) {
			$this->_user = $_COOKIE['wp_psr'];
		} else {
		  if (!isset($this->_user)) {
		    srand((double)microtime()*1234567);
  			$this->_user = md5(microtime() . rand(1000, 90000000));
		  }
		}
		setcookie('wp_psr', $this->_user, time()+60*60*24*365, '/');
		if (isset($_POST['psr_stars'])) {
			$points = (int) $_POST['psr_stars'];
			if (($points > 0) && ($points <= 5)) {
				$this->_points = $points;
			}
		}
	}
	
	/**
	 * Echo CSS style to render the stars
	 *
	 */
	function css()
	{
		$home = get_settings('home');
		echo "<style type=\"text/css\">
		.PSR_stars {
		  height: 15px;
		  overflow: hidden;
		  padding: 0;
		  margin: 0;
		}
		* html .PSR_stars .star {
			display: block;
			position: absolute;
			height: 0;
			width: 0;
		}
		.PSR_stars input {
			display: none;
		}
		.PSR_no_star, .PSR_half_star, .PSR_full_star {
		  display: block;
		  float: left;
		  width: 17px;
		  height: 15px;
		  text-indent: -1000em;
		  background-repeat: no-repeat;
		}
		.PSR_no_star {
		  background-image: url(" . $home . "/wp-content/plugins/post-star-rating/img/stars/no_star.gif);
		}
		.PSR_full_star {
		  background-image: url(" . $home . "/wp-content/plugins/post-star-rating/img/stars/full_star.gif);
		}
		.PSR_half_star {
		  background-image: url(" . $home . "/wp-content/plugins/post-star-rating/img/stars/half_star.gif);
		}
		.PSR_votes {
			padding-left: .5em;
		}
		</style>\n";
	}
	
	/**
	 * Echo javascript that generates the onMouseOver behaviour
	 *
	 */
	function js() {
		echo "<script type=\"text/javascript\">
		<!--
		function PSR_star_over(obj, star_number) {
			var psr=obj.parentNode;
			var as=psr.getElementsByTagName('label');
			for (i=0;i<star_number;++i) {
				as[i].className = 'PSR_full_star';
			}
			for (;i<as.length;++i) {
				as[i].className = 'PSR_no_star';
			}
		}
		function PSR_star_out(obj) {
			var as=obj.getElementsByTagName('label');
			for (i=0;i<as.length;++i) {
				as[i].className = 'PSR_no_star';
			}
		}
		//-->
		</script>\n";
	}
	
	/**
	 * Callback for Wordpress action wp_head.
	 * Echo CSS and Javascript
	 *
	 * @param unknown_type $unused
	 */
	function wp_head($unused) {
		$this->css();
		$this->js();
	}
}
?>